@@ -19,6 +19,9 @@ __PACKAGE__->result_source_instance->view_definition(<<ENDSQL
   UNION
   SELECT username, 'ldap' AS role FROM users
     WHERE ldap
+  UNION
+  SELECT username, 'api' AS role FROM users
+    WHERE token IS NOT NULL AND token_from IS NOT NULL
 ENDSQL
 );
 
